@@ -28,9 +28,7 @@
 void
 InitCard (char *name)
 {
-    KdCardAttr	attr;
-
-    KdCardInfoAdd (&fbdevFuncs, &attr, 0);
+    KdCardInfoAdd (&fbdevFuncs, 0);
 }
 
 void
@@ -42,17 +40,7 @@ InitOutput (ScreenInfo *pScreenInfo, int argc, char **argv)
 void
 InitInput (int argc, char **argv)
 {
-    KdKeyboardInfo *ki;
-
-    KdAddKeyboardDriver (&LinuxKeyboardDriver);
-    KdAddPointerDriver (&LinuxMouseDriver);
-#ifdef TSLIB
-    KdAddPointerDriver (&TsDriver);
-#endif
-
-    ki = KdParseKeyboard ("keybd");
-    KdAddKeyboard(ki);
-
+    KdOsAddInputDrivers ();
     KdInitInput ();
 }
 
@@ -70,11 +58,11 @@ ddxProcessArgument (int argc, char **argv, int i)
 {
   if (!strcmp (argv[i], "-fb"))
     {
-      if (i+1 < argc) 
+      if (i+1 < argc)
 	{
 	  fbdevDevicePath = argv[i+1];
 	  return 2;
-	} 
+	}
       UseMsg();
       exit(1);
     }
@@ -96,18 +84,18 @@ KdCardFuncs	fbdevFuncs = {
     fbdevRestore,	    /* restore */
     fbdevScreenFini,	    /* scrfini */
     fbdevCardFini,	    /* cardfini */
-    
+
     0,			    /* initCursor */
     0,			    /* enableCursor */
     0,			    /* disableCursor */
     0,			    /* finiCursor */
     0,			    /* recolorCursor */
-    
+
     0,			    /* initAccel */
     0,			    /* enableAccel */
     0,			    /* disableAccel */
     0,			    /* finiAccel */
-    
+
     fbdevGetColors,    	    /* getColors */
     fbdevPutColors,	    /* putColors */
 };
