@@ -29,7 +29,6 @@ KdCardInfo  *kdCardInfo;
 
 KdCardInfo *
 KdCardInfoAdd (KdCardFuncs  *funcs,
-	       KdCardAttr   *attr,
 	       void	    *closure)
 {
     KdCardInfo	*ci, **prev;
@@ -40,7 +39,6 @@ KdCardInfoAdd (KdCardFuncs  *funcs,
     for (prev = &kdCardInfo; *prev; prev = &(*prev)->next);
     *prev = ci;
     ci->cfuncs = funcs;
-    ci->attr = *attr;
     ci->closure = closure;
     ci->screenList = 0;
     ci->selected = 0;
@@ -138,34 +136,28 @@ KdFreePointer(KdPointerInfo *pi)
 {
     InputOption *option, *prev = NULL;
 
-    if (pi->name)
-        xfree(pi->name);
-    if (pi->path)
-        xfree(pi->path);
+    xfree(pi->name);
+    xfree(pi->path);
 
     for (option = pi->options; option; option = option->next) {
-        if (prev)
-            xfree(prev);
-        if (option->key)
-            xfree(option->key);
-        if (option->value)
-            xfree(option->value);
+        xfree(prev);
+        xfree(option->key);
+        xfree(option->value);
         prev = option;
     }
 
-    if (prev)
-        xfree(prev);
-    
+    xfree(prev);
     xfree(pi);
 }
- 
+
 void
 KdFreeKeyboard(KdKeyboardInfo *ki)
 {
-    if (ki->name)
-        xfree(ki->name);
-    if (ki->path)
-        xfree(ki->path);
+    xfree(ki->name);
+    xfree(ki->path);
+    xfree(ki->xkbRules);
+    xfree(ki->xkbModel);
+    xfree(ki->xkbLayout);
     ki->next = NULL;
     xfree(ki);
 }
